function [ab] = AdaBooster(cl)
% function [ab] = adaBooster(cl, nStages)
%    constructor of the adaBooster class that inherits from the classifier
%    class. the adaBooster implements the AdaBoost boosting algorithm to
%    build a strong (boosted) classifier from several weak classifiers.
%
%    Inputs:
%       cl: the weak classifier that needs to be boosted or another
%           adaBooster that was trained with a specific number of stages
%           and we need to increase them (default [])

% parameters needed for training
% the error bound after reaching which the classifier stops learning,
% used only when the nStages argument to learn is Inf
ab.errBound = 0.001;

% paramters needed to define the classifier
if nargin == 0
    ab.weakCl = [];
else
    ab.weakCl = cl;
end

% parameters to be learned by the classifier
% 1. a cell array of trained instances of the weak classifier
ab.trndCls = {};

% 2. the weight function that computes the output of adaBooster as a
% function of the outputs of the trained weak classifiers
ab.clsWeights = [];

% 3. example weights of the last iteration of adaBoost learning
% algorithm. this is saved after training the classifier to be used
% later if we want to increase the number of stages later on
ab.lastExWeights = [];

% 4. the threshold whereby the classifier can distinguish between the
% positive and negative examples
ab.thresh = NaN;

% 5. detection rate after training
ab.detectionRate = NaN;

ab = class(ab, 'AdaBooster', Classifier(getNumClasses(cl)));
