function [ab, clErr, resDetRate] = adjustThreshold(ab, trainingExamples, targetOuts, reqDetRate)
% function [ab, clErr] = adjustThreshold(ab, trainingExamples, targetOuts, reqDetRate)
%    computes the threshold that achives a specific detection rate along
%    with the associated classification error
%
%   Inputs:
%       trainingExamples: examples for training the classifier.
%       targetOuts: target classification outputs. its size must be the
%           same as the number of examples.
%       reqDetRate: required detection rate
%   Outputs:
%       ab: adaBooster with adjusted threshold
%       clErr: classification error
%       resDetRate: actual detection rate after adjustment

[clOuts, realOuts] = computeOutputs(ab, trainingExamples);
posVal = getPosVal(ab);
negVal = getNegVal(ab);
posIndxs = find(targetOuts == posVal);
numPos = length(posIndxs);
targetPos = ceil(numPos * reqDetRate);
numDetPos = sum(clOuts == posVal & targetOuts == posVal);
if targetPos > numDetPos,
    % the threshold needs to be adjusted to achieve the required detection
    % rate,
    cmpSign = sign(posVal - negVal);
    if cmpSign > 0,
        sortMode = 'ascend';
    else
        sortMode = 'descend';
    end
    sortedPosOuts = sort(realOuts(posIndxs), sortMode);
    ab.thresh = sortedPosOuts(numPos - targetPos + 1);
    clOuts = computeOutputs(ab, trainingExamples);
end
clErr = sum(clOuts ~= targetOuts) / length(clOuts);
resDetRate = sum((targetOuts == posVal) & (clOuts == posVal)) / ...
             sum((targetOuts == posVal));
ab.detectionRate = resDetRate;