function [outs, realOuts] = computeOutputs(ab, examples, extraArg)
% function [outs] = computeOutputs(ab, examples)
%   computes the classification outputs of adaBoost for the given examples
%
%   Inputs:
%       examples: set of example patterns to be classified
%       extraArg: an extra argument to be used if needed

if isempty(ab.weakCl)
    error('A weak classifier is not specified');
end
if isnan(ab.thresh) && ~ab.perfect
    error('AdaBoost classifier has not been trained');
end

if nargin == 1
    error('incorrect number of arguments');
elseif nargin == 2
    extraArgs = {examples};
else
    if iscell(extraArg),
        extraArgs = {examples, extraArg{:}}; %#ok<CCAT>
    else
        extraArgs = {examples, extraArg};
    end
end

wkClOuts = computeOutputs(ab.trndCls{1}, extraArgs{:});
nExamples = numel(wkClOuts);
realOuts = zeros(nExamples, getNumClasses(ab));
sz = size(realOuts);
rowInds = (1:nExamples)';
realOuts(sub2ind(sz, rowInds, wkClOuts)) = ab.clsWeights(1);
for i = 2:length(ab.clsWeights),
    wkClOuts = computeOutputs(ab.trndCls{i}, extraArgs{:});
    inds = sub2ind(sz, rowInds, wkClOuts);
    realOuts(inds) = realOuts(inds) + ab.clsWeights(i);
end
if getNumClasses(ab) == 2
    outsPos = (realOuts(:, getPosVal(ab)) - realOuts(:, getNegVal(ab))) >= ab.thresh;
    outs(outsPos) = getPosVal(ab);
    outs(~outsPos) = getNegVal(ab);
    outs = outs';
else
    [~, outs] = max(realOuts, [], 2);
end
