function [cl, clErr] = learn(cl, trnExamples, targetOuts, wts)
% function [cl, err] = learn(cl, trnExamples, targetOuts)
%   learning function for the BestDecisionStumpClassifier
%
%   Inputs:
%       trainingExamples: examples for training the classifier.
%       targetOuts: target classification outputs. its size must be the
%           same as the number of examples.
%       wts: weights of training examples (used to compute the weighted
%           classification error)
%   Outputs:
%       cl : trained BestDecisionStumpClassifier
%       clErr: classification error of the trained classifier

%% Deal with Cell Array Input
if iscell(trnExamples),
    trnExamples = cell2mat(trnExamples);
end

cl.trainedCl = ClassificationTree.fit(trnExamples, targetOuts, ...
    'weights', wts, 'MinParent', size(trnExamples,1), 'MergeLeaves', 'off');

view(cl.trainedCl);
cl.isTrained = true;

%% Compute Classification Error
if nargout > 1
    outs = cl.trainedCl.predict(trnExamples);

    if nargin < 4,
        clErr = sum(outs ~= targetOuts) / nExamples;
    else
        clErr = (outs ~= targetOuts)' * wts;
    end

    if isnan(clErr),
        error('error is nan');
    end
end
