function [lb] = LogitBooster(rg, numClasses)
% function [lb] = LogitBooster(rg)
%    constructor of the LogitBooster class that inherits from the classifier
%    class. the LogitBooster implements the LogitBoost boosting algorithm to
%    build a strong (boosted) classifier from several weak classifiers.
%
%    Inputs:
%       rg: the regressor needed to for logitboost
%       numClasses: number of classes


% parameters needed for training
% the error bound after reaching which the classifier stops learning,
% used only when the nStages argument to learn is Inf
lb.errBound = 0.001;

% paramters needed to define the classifier
if nargin == 0
    lb.regressor = Nan;
else
    lb.regressor = rg;
end

lb.nStages = 0;

% 1. number of classes
lb.numClasses = numClasses;

% 2. example weights of the last iteration of LogitBoost learning
% algorithm. this is saved after training the classifier to be used
% later if we want to increase the number of stages later on
lb.lastExWeights = [];

% 3. the threshold whereby the classifier can distinguish between the
% positive and negative examples
lb.thresh = NaN;

% 4. detection rate after training
lb.detectionRate = NaN;

% 5. F matrix (J,n) (j number of classes, n is the number of features)
lb.F = NaN;

% 6. P matrix (m, n) (m number of instances , n is the number of features)
lb.P = NaN;

lb = class(lb, 'LogitBooster', Classifier(numClasses));
