function [rab] = RealAdaBooster(cl)
% function [ab] = RealAdaBooster(cl, nStages)
%    constructor of the realAdaBooster class that inherits from the classifier
%    class. the realAdaBooster implements the RealAdaBoost boosting algorithm to
%    build a strong (boosted) classifier from several weak classifiers.
%
%    Inputs:
%       cl: the weak classifier that needs to be boosted or another
%           realAdaBooster that was trained with a specific number of stages
%           and we need to increase them (default [])

% parameters needed for training
% the error bound after reaching which the classifier stops learning,
% used only when the nStages argument to learn is Inf
rab.errBound = 0.001;

% paramters needed to define the classifier
if nargin == 0
    rab.weakCl = [];
    numClasses = 2;
else
    rab.weakCl = cl;
    numClasses = getNumClasses(cl);
end

% parameters to be learned by the classifier
% 1. a cell array of trained instances of the weak classifier
rab.trndCls = {};

% 2. Number of stages
rab.nStages = 0;

% 3. example weights of the last iteration of realAdaBoost learning
% algorithm. this is saved after training the classifier to be used
% later if we want to increase the number of stages later on
rab.lastExWeights = [];

% 4. Weak classifiers weights
rab.clsWeights = [];

% 5. the threshold whereby the classifier can distinguish between the
% positive and negative examples
rab.thresh = NaN;

% 6. detection rate after training
rab.detectionRate = NaN;

rab = class(rab, 'RealAdaBooster', Classifier(numClasses));
