function [ features ] = getRankedFeatures( cl )
%GETRANKEDFEATURES return each feature used with its score
%   Inputs:
%       cl : real adaboost instance
%   Outputs:
%       features: featureInfo struct sorted descendingly by score
    %% retrieve features from weak classifier
    allFeatures = [];
    
    weakCls = cl.trndCls;
    scores = abs(cl.clsWeights);
    
    for i=1:length(weakCls)
        wcl = weakCls{i};
        wf = getFeaturesInfo(wcl);
        % assuming that weak classifier can emmit more than one feature
        for j=1:length(wf)
            wf(j).score = scores(i);
        end
        allFeatures = [allFeatures; wf];
    end
    
    %% get Accumulated score for each feature
    %sum features which have the same id;
    [~, ind] = sort([allFeatures.id]);
    allFeatures = allFeatures(ind);
    
    
    features = [];
    s = allFeatures(1).score;
    count = 1;
    for i=2:length(allFeatures)    
        if allFeatures(i).id ~= allFeatures(i-1).id
            allFeatures(i-1).score = s;
            allFeatures(i-1).count = count;
            features = [features; allFeatures(i-1)];
            s = 0;
            count = 0;
        end
        s = s + allFeatures(i).score;
        count = count + 1;
    end
    allFeatures(end).score = s;
    allFeatures(end).count = count;
    features = [features; allFeatures(end)];
    
    %% rank features based on score
    [~, ind]=  sort([features.score], 'descend');
    features = features(ind);
    
end

