function [ tcl ] = loadFromFid( cl, fid )
%LOADFROMFID load classifier from binary file
%   Load RealAda boost classifier from binary file
%
%   Inputs:
%       cl: dummy realAda classifier object (RealAdaBooster)
%       fid : file descriptor
%   Outputs:
%       tcl: instance of RealAdaboost classifier

%% Read weak classifier name
[nameLength, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end
[weakClNameData, count] = fread(fid, nameLength, '*uint8');
if count <= 0
    error('Error reading from file\n');
end
weakClName = deserialize(weakClNameData);
weakCl = eval(weakClName);

%% Read err bound
[tcl.errBound, count] = fread(fid, 1, 'double');
if count <= 0
    error('Error reading from file\n');
end
%% Read untrained weak classifier
tcl.weakCl = loadFromFid(weakCl,fid);
%% Read trained weak classifiers

[numWeakClassifiers, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end
tcl.trndCls = cell(1, numWeakClassifiers);

% read classifier data
for i = 1:numWeakClassifiers
    tcl.trndCls{i} = loadFromFid(weakCl, fid);
end

%% Read number of stages
[tcl.nStages, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end

%% Read last instances weights

% read length of weights
[dataSize, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end
% read weights array
[data, count] = fread(fid, dataSize, '*uint8');
if count <= 0
    error('Error reading from file\n');
end
tcl.lastExWeights = deserialize(data);
%% Read cls weights
% read length of weights
[dataSize, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end
% read weights array
[data, count] = fread(fid, dataSize, '*uint8');
if count <= 0
    error('Error reading from file\n');
end
tcl.clsWeights = deserialize(data);

%% read threshold
[tcl.thresh, count] = fread(fid, 1, 'double');
if count <= 0
    error('Error reading from file\n');
end
%% write detection rate
[tcl.detectionRate, count] = fread(fid, 1, 'double');
if count <= 0
    error('Error reading from file\n');
end

%% create real ada object
tcl = class(tcl, 'RealAdaBooster', Classifier(getNumClasses(weakCl)));

end

