function [outs, prob] = computeOutputs(svmCl, examples)
% function [outs] = computeOutputs(svmCl, examples)
%   computes the classification outputs for the given examples
%
%   Inputs:
%       svmCl: trained svm classifier.
%       examples: instance to be classified.
%                 Number of Instances X Number of Features
%
%   Outputs:
%       outs: predicted classification outputs
%       prob: probability estimates (a probability of each estimate
%             classified to belong to the selected class)

if ~isstruct(svmCl.trainedSVM),
    error('classifier is not trained');
end

%% Handle Cell Arrays
if iscell(examples),
    examples = cell2mat(examples);
end

%% Compute Outs
dummy = ones(size(examples, 1), 1);

[outs, ~, p] = svmpredict(dummy, examples, svmCl.trainedSVM, svmCl.libSvmPrdOpts);

prob = NaN;
%fix returned probabilities (re arrange them) Only if -b option was used
if numel(strfind(svmCl.libSvmPrdOpts,'-b 1')) ~= 0
    model = svmCl.trainedSVM;
    map = zeros(getNumClasses(svmCl),1);
    map(model.Label) = (1:getNumClasses(svmCl))';
    prob = p(:,map);
end
