function [ cl ] = loadFromFid( dummy, fid )
%LOADFROMFID loads svm classifier from binary file
%   Inputs:
%       dummy : svm classifier dummy object (SVMClassifier)
%       fid : file descriptor
%   Outputs:
%       cl : loaded svm classifier object

% read number of classes
[numClasses, count] = fread(fid, 1, 'uint16');
if count <= 0
    error('Error reading from file\n');
end

%% Read Classifier object

% read length of classifier object
[clSize, count] = fread(fid, 1, 'uint16');

if count <= 0
    error('Error reading from file\n');
end

% read classifier data
[clobj, count] = fread(fid, clSize, '*uint8');

if count <= 0
    error('Error reading from file\n');
end

cl.trainedSVM = deserialize(clobj);

if isstruct(cl.trainedSVM)
    %return any sparse values to its original state
    cl.trainedSVM.SVs = sparse(cl.trainedSVM.SVs);
end

%% Read libsvm training Classifier parameters

% read length of training parameters
[datasize, count] = fread(fid, 1, 'uint16');

if count <= 0
    error('Error reading from file\n');
end

% read classifier data
[data, count] = fread(fid, datasize, '*uint8');

if count <= 0
    error('Error reading from file\n');
end

cl.libSvmTrnOpts = deserialize(data);

%% Read libsvm training Classifier parameters
% read length of training parameters
[datasize, count] = fread(fid, 1, 'uint16');

if count <= 0
    error('Error reading from file\n');
end

% read classifier data
[data, count] = fread(fid, datasize, '*uint8');

if count <= 0
    error('Error reading from file\n');
end

cl.libSvmPrdOpts = deserialize(data);

%% Create new Object
cl = class(cl, 'SVMClassifier', Classifier(numClasses));
end