%% RealAda Classifier Demo
% Several ways to use RealAda Boosting Algorithm classifier
%%
%% Load dataset

% Name of matlab dataset
% Available names : [cancer, wine, iris, crab, glass, simpleclass, thyroid]
datasetName = 'iris';

% This is just an example to load matlab datasets
% you can load datasets from different sources with different ways
% as long as the you provide x the training instance which is a matrix
% of size(Number of Instance,Number of Features) and y which is the
% label matrix having size(Number of Instance, 1)
load(strcat(datasetName, '_dataset'));
eval(sprintf('x = %sInputs;', datasetName));
eval(sprintf('y = %sTargets;', datasetName));
x = x';
y = y';
numClasses = size(y, 2);
[~,y] = max(y,[],2);

numFeatures = size(x, 2);
numInstances = size(x, 1);

% display dataset info
disp(['Dataset Name ' datasetName]);
disp(['Number of Classes ' num2str(numClasses)]);
disp(['Number of Instances ' num2str(numInstances)]);
disp(['Number of Features ' num2str(numFeatures)]);


%% Basic Usage

fprintf('===========\n');
fprintf('Basic Usage\n');
fprintf('===========\n');

% create RealAdaBoost classifier Using svm as weak classifier
% NOTE : weak classifier must support probability estimates and
% SVMClassifier supports probability estimates by bassing -b argument
radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));

% train RealAda classifier
fprintf('\tTraining Classifier for 3 iterations\n');
[radacl, learnErr] = learn(radacl, x, y, 3);
fprintf('\tLearning Error %f\n', learnErr);

fprintf('\t-----\n');
fprintf('\tTesting Classifier\n');
outs = computeOutputs(radacl, x);
% other way to calculate error either on training dataset or other dataset
err = sum(outs ~= y) / numInstances;
fprintf('\tLearning Error %f\n', err);

% comparing outputs of first 5 instances in predicted and target outputs
fprintf('\t[predicted outputs : correct outputs]\n');
fprintf('\t\t%d\t\t%d\t\n',outs(1:5, :), y(1:5 , :));

%% Stage Details

fprintf('=============\n');
fprintf('Stage Details\n');
fprintf('=============\n');

radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));

radacl = learn(radacl, x, y, 3, true);
fprintf('\tLearning Error %f\n', learnErr);

%% Display Classifier

fprintf('==================\n');
fprintf('Display Classifier\n');
fprintf('==================\n');

fprintf('\tDisplay Classifier Before Learning\n\t===>\n');
radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));
display(radacl);
fprintf('\t<===\n');

fprintf('\tDisplay Classifier After Learning\n===>\n');
radacl = learn(radacl, x, y, 2);
display(radacl);
fprintf('\t<===\n');

%% Iteratins Errors

fprintf('================\n');
fprintf('Iterations Error\n');
fprintf('================\n');

radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));

fprintf('\tTraining Classifier for 10 iterations\n');
[radacl, learnErr, iterationsErrors] = learn(radacl, x, y, 10);
fprintf('\tLearning Error %f\n', learnErr);

fprintf('\t------------\n');
fprintf('\tPlotting Iteration Errors\n');
plot(1:length(iterationsErrors), iterationsErrors);
ylim([0 1]);

%% Add More Boosting Stages

fprintf('========================\n');
fprintf('Add More Boosting Stages\n');
fprintf('========================\n');

radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));

fprintf('\tTraining Classifier for 5 iterations\n');
radacl = learn(radacl, x, y, 5);
fprintf('\t------------\n');
fprintf('\tTraining Classifier for 5 more iterations\n');
[radacl, learnErr] = learn(radacl, x, y, 10);
fprintf('\tLearning Error %f\n', learnErr);


%% Using Different Weak Classifier

fprintf('===============================\n');
fprintf('Using Different Weak Classifier\n');
fprintf('===============================\n');

radacl = RealAdaBooster(DecisionTreeClassifier(numClasses));

% train RealAda classifier
fprintf('\tTraining Classifier for 3 iterations\n');
[radacl, learnErr] = learn(radacl, x, y, 3);
fprintf('\tLearning Error %f\n', learnErr);


%% Saving and Loading classifier to and from a file

fprintf('=======================================\n');
fprintf('Save and Load Classifier to/from a file\n');
fprintf('=======================================\n');

radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));
[radacl, learnErr] = learn(radacl, x, y, 1);
fprintf('\tLearning Error %f\n', learnErr);
saveToFile(radacl, 'test2.bin');
radacl2 = loadFromFile(RealAdaBooster,'test2.bin');

outs1 = computeOutputs(radacl, x);
err1 = sum(outs1 ~= y) / numInstances;
outs2 = computeOutputs(radacl2, x);
err2 = sum(outs2 ~= y) / numInstances;

fprintf('\tError Before Save %f, Error After Save %f\n', err1, err2);

%% Add Boost Stages till reaching a required Err Bound

fprintf('===================================================\n');
fprintf('Add Boost Stages till reaching a required Err Bound\n');
fprintf('===================================================\n');

radacl = RealAdaBooster(SVMClassifier(numClasses, '-b 1', '-b 1'));
[radacl, learnErr] = learn(radacl, x, y, Inf, true, '', '', NaN, 0.02);
fprintf('\tLearning Error %f\n', learnErr);
