%% RealAda Classifier Demo 2
% Graphical representation of boosting iterations using Decision stumps
%%
%% Create synthetic data

numPoints = 300;
r1 = 10;
r2 = 5;
noiseRatio = 0.3;

nc1x1 =  r1 * (2 * noiseRatio * rand(numPoints/2,1) - noiseRatio);
nc1x2 =  r1 * (2 * noiseRatio * rand(numPoints/2,1) - noiseRatio);

nc2x1 =  r2 * (2 * noiseRatio * rand(numPoints/2,1) - noiseRatio);
nc2x2 =  r2 * (2 * noiseRatio * rand(numPoints/2,1) - noiseRatio);


angs1 = 2*pi*rand(numPoints/2, 1);
angs2 = 2*pi*rand(numPoints/2, 1);

x1 = [r1 * cos(angs1) +  nc1x1; r2 * cos(angs2) +  nc2x1];
x2 = [r1 * sin(angs1) +  nc1x2; r2 * sin(angs2) +  nc2x2];
x = [x1 x2];
y = [ones(numPoints/2,1) ; 2*ones(numPoints/2,1)];
c1 = y == 1;
c2 = y == 2;

figure(1);
clf;
hold on;
scatter(x1(c1), x2(c1),'b','o' ,'markerfacecolor','blue');
scatter(x1(c2), x2(c2),'b','+','markerfacecolor','blue');
title('Original');

numClasses = 2;


%% Run RealAdaBoost on Data

hFig = figure(2);
clf;
set(hFig, 'Position', [100 100 900 900])

stump = BestDecisionStumpClassifier(numClasses);
radacl = RealAdaBooster(stump);

for i=1:6
    [radacl, learnErr] = learn(radacl, x, y, i);
    fprintf('Error %f\n', learnErr);
    outs = computeOutputs(radacl, x);
    fprintf('Miss classified %d / %d\n', sum(y~=outs) , numPoints);
    
    c1c = y == 1 & y == outs;
    c1w = y == 1 & y ~= outs;
    c2c = y == 2 & y == outs;
    c2w = y == 2 & y ~= outs;

    subplot(2, 3, i);
    hold on;
    
    S = getWeights(radacl) * 5000;
    
    scatter(x1(c1c), x2(c1c),S(c1c),'g','o', 'markerfacecolor','g');
    scatter(x1(c1w), x2(c1w),S(c1w),'r','o', 'markerfacecolor','r');

    scatter(x1(c2c), x2(c2c),S(c2c),'g','+', 'markerfacecolor', 'g');
    scatter(x1(c2w), x2(c2w),S(c2w),'r','+', 'markerfacecolor' ,'r');
    title(['Iteration ' num2str(i)]);
   
    weakCls = getWeakCls(radacl);
    for j=1:length(weakCls)
        featureInfo = getFeaturesInfo(weakCls{j});
        feature = featureInfo.id;
        splitVal = featureInfo.splitVal;
        limit = r1 + noiseRatio*r1;
        if feature == 1
            line([splitVal splitVal], [-limit, limit]);
        else
            line([-limit, limit], [splitVal splitVal]);
        end
    end
    
    fprintf('miss Class 1 : %d / %d\n', sum(c1w), sum(c1c+c1w));
    fprintf('miss Class 2 : %d / %d\n', sum(c2w), sum(c2c+c2w));
    fprintf('-----\n');
end