%% SVM Classifier Demo
% Several ways to use svm classifier
%%
%% Load dataset

% Name of matlab dataset
% Available names : [cancer, wine, iris, crab, glass, simpleclass, thyroid]
datasetName = 'iris';

% This is just an example to load matlab datasets
% you can load datasets from different sources with different ways
% as long as the you provide x the training instance which is a matrix
% of size(Number of Instance,Number of Features) and y which is the
% label matrix having size(Number of Instance, 1)
load(strcat(datasetName, '_dataset'));
eval(sprintf('x = %sInputs;', datasetName));
eval(sprintf('y = %sTargets;', datasetName));
x = x';
y = y';
numClasses = size(y, 2);
[~,y] = max(y,[],2);

numFeatures = size(x, 2);
numInstances = size(x, 1);

% display dataset info
disp(['Dataset Name ' datasetName]);
disp(['Number of Classes ' num2str(numClasses)]);
disp(['Number of Instances ' num2str(numInstances)]);
disp(['Number of Features ' num2str(numFeatures)]);


%% Basic Usage

fprintf('===========\n');
fprintf('Basic Usage\n');
fprintf('===========\n');

% create svm classifier
svmcl = SVMClassifier(numClasses);

% train svm classifier
disp('Training Classifier');
[svmcl, learnErr] = learn(svmcl, x, y);
fprintf('Learning Error %f\n', learnErr);

disp('Testing Classifier');
outs = computeOutputs(svmcl, x);
% other way to calculate error either on learning dataset or other dataset
err = sum(outs ~= y) / numInstances;
fprintf('Learning Error %f\n', err);

% comparing outputs of first 5 instances in predicted and target outputs
disp('[predicted outputs : correct outputs]');
disp([outs(1:5, :) y(1:5 , :)]);
%% Instances Weights

fprintf('=================\n');
fprintf('Instances Weights\n');
fprintf('=================\n');

svmcl = SVMClassifier(numClasses);
wts = ones(numInstances, 1) / numInstances;

% training svm with given weights
svmcl = learn(svmcl, x, y, wts);

outs = computeOutputs(svmcl, x);

err = sum(outs ~= y) / numInstances;
fprintf('Error %f\n', err);

%% Passing arguments and probability estimates

fprintf('====================================\n');
fprintf('Passing arguments and prob estimates\n');
fprintf('====================================\n');
% -b 1 is argument used to get probability estimates (prob. that each
% instance belong to the predicted class)
% for complete list of argument see http://www.csie.ntu.edu.tw/~cjlin/libsvm/
svmcl = SVMClassifier(numClasses,'-c 10 -g 1 -b 1','-b 1');
svmcl = learn(svmcl, x, y);

[outs, prob] = computeOutputs(svmcl, x);

disp('[predicted output : correct output : class probabilities]');
disp([outs(1:5, :) y(1:5 , :) prob(1:5, :) ]);


%% Displaying The Classifier

fprintf('=========================\n');
fprintf('Displaying The Classifier\n');
fprintf('=========================\n');


svmcl = SVMClassifier(numClasses);

disp('Display before training');
display(svmcl);

disp('-----------------------');

svmcl = learn(svmcl, x, y);
disp('Display after training');
display(svmcl);



%% Save And Load Classifier To/From A File

fprintf('=======================================\n');
fprintf('Save And Load Classifier To/From A File\n');
fprintf('=======================================\n');

svmcl = SVMClassifier(numClasses);
svmcl = learn(svmcl, x, y, wts);

% same classifier to file
saveToFile(svmcl, 'test.bin');

% load classifier from file
svmcl2 = loadFromFile(SVMClassifier, 'test.bin');

outs1 = computeOutputs(svmcl, x);
err1 = sum(outs1 ~= y) / numInstances;
outs2 = computeOutputs(svmcl2, x);
err2 = sum(outs2 ~= y) / numInstances;

fprintf('Error Before Save %f, Error After Save %f\n', err1, err2);

%% Using kfold with svm
fprintf('=========================\n');
fprintf('Using Kfold with SVM\n');
fprintf('=========================\n');

svmcl = SVMClassifier(numClasses);
cp = kfold(x, y, 10, svmcl);

fprintf('Accuracy of 10 fold-cross validation %f\n', cp.CorrectRate * 100);
