function [ cp ] = kfold( x, y, k, cl, extraArgs )
%KFOLD Kfold cross validation for any classifier that
%      inherits from the classifier class

%  Inputs:
%    x: trainingExamples * numberOfFeatures
%        whole dataset
%    y: trainingExamples * 1
%        target outputs
%    cl: classifier object
%    extraArgs: any extra arguments

if (k < 1)
    error('invalid k')
end

if nargin < 4
    error('invalid no of arguments')
end

if nargin < 5
    extraArgs = {};
end
    
idx = crossvalind('Kfold', y, k);
cp = classperf(y);
for i = 1:k
   fprintf('FOLD %d\n', i);

   test = (idx == i); train = ~test;

   trainData = x(train,:);
   testData = x(test,:);
   outTrain = y(train,:);
   
   fprintf('\tLearn Size %d\n', size(trainData, 1));
   fprintf('\tTest Size %d\n', size(testData, 1));
   
   if (size(testData) == 0)
       error('choose larger value for k');
   end
   clcp = cl;
   clcp = learn(clcp, trainData, outTrain, extraArgs{:});
   outs = computeOutputs(clcp, testData);
   
   classperf(cp, outs, test);
   
   %===
   outTest = y(test, :);
   correct = sum(outs == outTest);
   fprintf('\n\t\tFOLD RESULT = %d / %d = %f\n', correct, size(outTest, 1), ...
       correct / size(outTest, 1) * 100);
   %===
end

end

